# Generated by Django 2.0 on 2017-12-30 13:16

import django.contrib.sites.managers
import django.db.models.deletion
import django.db.models.manager
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("sites", "0002_alter_domain_unique"),
        ("edc_identifier", "0012_auto_20171116_1606"),
    ]

    operations = [
        migrations.RemoveField(model_name="historicalidentifierhistory", name="history_user"),
        migrations.RemoveField(model_name="historicalidentifiertracker", name="history_user"),
        migrations.RemoveField(model_name="historicalsubjectidentifier", name="history_user"),
        migrations.DeleteModel(name="IdentifierHistory"),
        migrations.DeleteModel(name="IdentifierTracker"),
        migrations.DeleteModel(name="Sequence"),
        migrations.DeleteModel(name="SubjectIdentifier"),
        migrations.AlterModelManagers(
            name="identifiermodel",
            managers=[
                ("objects", django.db.models.manager.Manager()),
                ("on_site", django.contrib.sites.managers.CurrentSiteManager()),
            ],
        ),
        migrations.RenameField(
            model_name="identifiermodel",
            old_name="subject_type",
            new_name="identifier_type",
        ),
        migrations.RemoveField(model_name="identifiermodel", name="study_site"),
        migrations.AddField(
            model_name="identifiermodel",
            name="identifier_prefix",
            field=models.CharField(max_length=25, null=True),
        ),
        migrations.AddField(
            model_name="identifiermodel",
            name="site",
            field=models.ForeignKey(
                editable=False,
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                to="sites.Site",
            ),
        ),
        migrations.AlterField(
            model_name="identifiermodel",
            name="sequence_number",
            field=models.IntegerField(default=1),
        ),
        migrations.DeleteModel(name="HistoricalIdentifierHistory"),
        migrations.DeleteModel(name="HistoricalIdentifierTracker"),
        migrations.DeleteModel(name="HistoricalSubjectIdentifier"),
    ]



# Generated by Django 2.0.1 on 2018-01-16 10:35

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [("edc_identifier", "0014_auto_20180103_1322")]

    operations = [migrations.AlterModelManagers(name="identifiermodel", managers=[])]



from django.apps import apps as django_apps
from edc_utils import get_utcnow

from .research_identifier import ResearchIdentifier


class SubjectIdentifier(ResearchIdentifier):

    template: str = "{protocol_number}-{site_id}{device_id}{sequence}"
    label: str = "subjectidentifier"
    padding: int = 4

    def __init__(self, last_name: str = None, **kwargs):
        self.last_name = last_name
        super().__init__(**kwargs)

    def pre_identifier(self) -> None:
        pass

    def post_identifier(self) -> None:
        """Creates a registered subject instance for this
        subject identifier.
        """
        model = django_apps.get_app_config("edc_registration").model
        model.objects.create(
            subject_identifier=self.identifier,
            site=self.site,
            subject_type=self.identifier_type,
            last_name=self.last_name,
            registration_datetime=get_utcnow(),
        )



# Generated by Django 2.0.1 on 2018-01-28 08:54

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [("edc_identifier", "0017_identifiermodel_subject_identifier")]

    operations = [
        migrations.AlterField(
            model_name="identifiermodel",
            name="protocol_number",
            field=models.CharField(max_length=25, null=True),
        )
    ]



# Generated by Django 2.0b1 on 2017-11-16 14:06

import _socket
import edc_model_fields.fields.userfield
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [("edc_identifier", "0011_auto_20170511_1323")]

    operations = [
        migrations.AlterModelOptions(
            name="historicalidentifierhistory",
            options={
                "get_latest_by": "history_date",
                "ordering": ("-history_date", "-history_id"),
                "verbose_name": "historical ",
            },
        ),
        migrations.AlterModelOptions(
            name="historicalidentifiertracker",
            options={
                "get_latest_by": "history_date",
                "ordering": ("-history_date", "-history_id"),
                "verbose_name": "historical ",
            },
        ),
        migrations.AlterModelOptions(
            name="historicalsubjectidentifier",
            options={
                "get_latest_by": "history_date",
                "ordering": ("-history_date", "-history_id"),
                "verbose_name": "historical ",
            },
        ),
        migrations.AddField(
            model_name="historicalidentifierhistory",
            name="device_created",
            field=models.CharField(blank=True, max_length=10),
        ),
        migrations.AddField(
            model_name="historicalidentifierhistory",
            name="device_modified",
            field=models.CharField(blank=True, max_length=10),
        ),
        migrations.AddField(
            model_name="historicalidentifierhistory",
            name="history_change_reason",
            field=models.CharField(max_length=100, null=True),
        ),
        migrations.AddField(
            model_name="historicalidentifiertracker",
            name="device_created",
            field=models.CharField(blank=True, max_length=10),
        ),
        migrations.AddField(
            model_name="historicalidentifiertracker",
            name="device_modified",
            field=models.CharField(blank=True, max_length=10),
        ),
        migrations.AddField(
            model_name="historicalidentifiertracker",
            name="history_change_reason",
            field=models.CharField(max_length=100, null=True),
        ),
        migrations.AddField(
            model_name="historicalsubjectidentifier",
            name="device_created",
            field=models.CharField(blank=True, max_length=10),
        ),
        migrations.AddField(
            model_name="historicalsubjectidentifier",
            name="device_modified",
            field=models.CharField(blank=True, max_length=10),
        ),
        migrations.AddField(
            model_name="historicalsubjectidentifier",
            name="history_change_reason",
            field=models.CharField(max_length=100, null=True),
        ),
        migrations.AddField(
            model_name="identifierhistory",
            name="device_created",
            field=models.CharField(blank=True, max_length=10),
        ),
        migrations.AddField(
            model_name="identifierhistory",
            name="device_modified",
            field=models.CharField(blank=True, max_length=10),
        ),
        migrations.AddField(
            model_name="identifiermodel",
            name="device_created",
            field=models.CharField(blank=True, max_length=10),
        ),
        migrations.AddField(
            model_name="identifiermodel",
            name="device_modified",
            field=models.CharField(blank=True, max_length=10),
        ),
        migrations.AddField(
            model_name="identifiertracker",
            name="device_created",
            field=models.CharField(blank=True, max_length=10),
        ),
        migrations.AddField(
            model_name="identifiertracker",
            name="device_modified",
            field=models.CharField(blank=True, max_length=10),
        ),
        migrations.AddField(
            model_name="subjectidentifier",
            name="device_created",
            field=models.CharField(blank=True, max_length=10),
        ),
        migrations.AddField(
            model_name="subjectidentifier",
            name="device_modified",
            field=models.CharField(blank=True, max_length=10),
        ),
        migrations.AlterField(
            model_name="historicalidentifierhistory",
            name="hostname_created",
            field=models.CharField(
                blank=True,
                default=_socket.gethostname,
                help_text="System field. (modified on create only)",
                max_length=60,
            ),
        ),
        migrations.AlterField(
            model_name="historicalidentifierhistory",
            name="user_created",
            field=edc_model_fields.fields.userfield.UserField(
                blank=True,
                help_text="Updated by admin.save_model",
                max_length=50,
                verbose_name="user created",
            ),
        ),
        migrations.AlterField(
            model_name="historicalidentifierhistory",
            name="user_modified",
            field=edc_model_fields.fields.userfield.UserField(
                blank=True,
                help_text="Updated by admin.save_model",
                max_length=50,
                verbose_name="user modified",
            ),
        ),
        migrations.AlterField(
            model_name="historicalidentifiertracker",
            name="hostname_created",
            field=models.CharField(
                blank=True,
                default=_socket.gethostname,
                help_text="System field. (modified on create only)",
                max_length=60,
            ),
        ),
        migrations.AlterField(
            model_name="historicalidentifiertracker",
            name="user_created",
            field=edc_model_fields.fields.userfield.UserField(
                blank=True,
                help_text="Updated by admin.save_model",
                max_length=50,
                verbose_name="user created",
            ),
        ),
        migrations.AlterField(
            model_name="historicalidentifiertracker",
            name="user_modified",
            field=edc_model_fields.fields.userfield.UserField(
                blank=True,
                help_text="Updated by admin.save_model",
                max_length=50,
                verbose_name="user modified",
            ),
        ),
        migrations.AlterField(
            model_name="historicalsubjectidentifier",
            name="hostname_created",
            field=models.CharField(
                blank=True,
                default=_socket.gethostname,
                help_text="System field. (modified on create only)",
                max_length=60,
            ),
        ),
        migrations.AlterField(
            model_name="historicalsubjectidentifier",
            name="user_created",
            field=edc_model_fields.fields.userfield.UserField(
                blank=True,
                help_text="Updated by admin.save_model",
                max_length=50,
                verbose_name="user created",
            ),
        ),
        migrations.AlterField(
            model_name="historicalsubjectidentifier",
            name="user_modified",
            field=edc_model_fields.fields.userfield.UserField(
                blank=True,
                help_text="Updated by admin.save_model",
                max_length=50,
                verbose_name="user modified",
            ),
        ),
        migrations.AlterField(
            model_name="identifierhistory",
            name="hostname_created",
            field=models.CharField(
                blank=True,
                default=_socket.gethostname,
                help_text="System field. (modified on create only)",
                max_length=60,
            ),
        ),
        migrations.AlterField(
            model_name="identifierhistory",
            name="user_created",
            field=edc_model_fields.fields.userfield.UserField(
                blank=True,
                help_text="Updated by admin.save_model",
                max_length=50,
                verbose_name="user created",
            ),
        ),
        migrations.AlterField(
            model_name="identifierhistory",
            name="user_modified",
            field=edc_model_fields.fields.userfield.UserField(
                blank=True,
                help_text="Updated by admin.save_model",
                max_length=50,
                verbose_name="user modified",
            ),
        ),
        migrations.AlterField(
            model_name="identifiermodel",
            name="hostname_created",
            field=models.CharField(
                blank=True,
                default=_socket.gethostname,
                help_text="System field. (modified on create only)",
                max_length=60,
            ),
        ),
        migrations.AlterField(
            model_name="identifiermodel",
            name="user_created",
            field=edc_model_fields.fields.userfield.UserField(
                blank=True,
                help_text="Updated by admin.save_model",
                max_length=50,
                verbose_name="user created",
            ),
        ),
        migrations.AlterField(
            model_name="identifiermodel",
            name="user_modified",
            field=edc_model_fields.fields.userfield.UserField(
                blank=True,
                help_text="Updated by admin.save_model",
                max_length=50,
                verbose_name="user modified",
            ),
        ),
        migrations.AlterField(
            model_name="identifiertracker",
            name="hostname_created",
            field=models.CharField(
                blank=True,
                default=_socket.gethostname,
                help_text="System field. (modified on create only)",
                max_length=60,
            ),
        ),
        migrations.AlterField(
            model_name="identifiertracker",
            name="user_created",
            field=edc_model_fields.fields.userfield.UserField(
                blank=True,
                help_text="Updated by admin.save_model",
                max_length=50,
                verbose_name="user created",
            ),
        ),
        migrations.AlterField(
            model_name="identifiertracker",
            name="user_modified",
            field=edc_model_fields.fields.userfield.UserField(
                blank=True,
                help_text="Updated by admin.save_model",
                max_length=50,
                verbose_name="user modified",
            ),
        ),
        migrations.AlterField(
            model_name="sequence",
            name="hostname_created",
            field=models.CharField(
                blank=True,
                default=_socket.gethostname,
                help_text="System field. (modified on create only)",
                max_length=60,
            ),
        ),
        migrations.AlterField(
            model_name="sequence",
            name="user_created",
            field=edc_model_fields.fields.userfield.UserField(
                blank=True,
                help_text="Updated by admin.save_model",
                max_length=50,
                verbose_name="user created",
            ),
        ),
        migrations.AlterField(
            model_name="sequence",
            name="user_modified",
            field=edc_model_fields.fields.userfield.UserField(
                blank=True,
                help_text="Updated by admin.save_model",
                max_length=50,
                verbose_name="user modified",
            ),
        ),
        migrations.AlterField(
            model_name="subjectidentifier",
            name="hostname_created",
            field=models.CharField(
                blank=True,
                default=_socket.gethostname,
                help_text="System field. (modified on create only)",
                max_length=60,
            ),
        ),
        migrations.AlterField(
            model_name="subjectidentifier",
            name="user_created",
            field=edc_model_fields.fields.userfield.UserField(
                blank=True,
                help_text="Updated by admin.save_model",
                max_length=50,
                verbose_name="user created",
            ),
        ),
        migrations.AlterField(
            model_name="subjectidentifier",
            name="user_modified",
            field=edc_model_fields.fields.userfield.UserField(
                blank=True,
                help_text="Updated by admin.save_model",
                max_length=50,
                verbose_name="user modified",
            ),
        ),
    ]



from django.db import models


class Enrollment(models.Model):
    subject_identifier = models.CharField(max_length=25, null=True)


class EnrollmentThree(models.Model):
    subject_identifier = models.CharField(max_length=25, null=True)



import re

from edc_protocol import Protocol

from .exceptions import SubjectIdentifierError


def is_subject_identifier_or_raise(subject_identifier, reference_obj=None, raise_on_none=None):
    """Returns the given subject identifier.

    * If the format of the `subject_identifier` is invalid,
      raises an exception.
    * If `subject_identifier` is None, does nothing, unless
      `raise_on_none` is `True`.
    """
    if subject_identifier or raise_on_none:
        if not re.match(Protocol().subject_identifier_pattern, subject_identifier or ""):
            reference_msg = ""
            if reference_obj:
                reference_msg = f"See {repr(reference_obj)}. "
            raise SubjectIdentifierError(
                f"Invalid format for subject identifier. {reference_msg}"
                f"Got `{subject_identifier or ''}`. "
                f"Expected pattern `{Protocol().subject_identifier_pattern}`"
            )
    return subject_identifier



# -*- coding: utf-8 -*-
# Generated by Django 1.10.3 on 2016-11-27 22:26
from __future__ import unicode_literals

import django_audit_fields.fields.uuid_auto_field
import django_revision.revision_field
import edc_model_fields.fields.hostname_modification_field
import edc_model_fields.fields.userfield
import edc_utils
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        (
            "edc_identifier",
            "0005_historicalidentifierhistory_historicalidentifiertracker_historicalsubjectidentifier",
        )
    ]

    operations = [
        migrations.AddField(
            model_name="historicalidentifierhistory",
            name="created",
            field=models.DateTimeField(default=edc_utils.date.get_utcnow, editable=False),
        ),
        migrations.AddField(
            model_name="historicalidentifierhistory",
            name="hostname_created",
            field=models.CharField(
                default="mac2-2.local",
                editable=False,
                help_text="System field. (modified on create only)",
                max_length=50,
            ),
        ),
        migrations.AddField(
            model_name="historicalidentifierhistory",
            name="hostname_modified",
            field=edc_model_fields.fields.hostname_modification_field.HostnameModificationField(
                blank=True,
                editable=False,
                help_text="System field. (modified on every save)",
                max_length=50,
            ),
        ),
        migrations.AddField(
            model_name="historicalidentifierhistory",
            name="modified",
            field=models.DateTimeField(default=edc_utils.date.get_utcnow, editable=False),
        ),
        migrations.AddField(
            model_name="historicalidentifierhistory",
            name="revision",
            field=django_revision.revision_field.RevisionField(
                blank=True,
                editable=False,
                help_text="System field. Git repository tag:branch:commit.",
                max_length=75,
                null=True,
                verbose_name="Revision",
            ),
        ),
        migrations.AddField(
            model_name="historicalidentifierhistory",
            name="user_created",
            field=edc_model_fields.fields.userfield.UserField(
                blank=True, editable=False, max_length=50, verbose_name="user created"
            ),
        ),
        migrations.AddField(
            model_name="historicalidentifierhistory",
            name="user_modified",
            field=edc_model_fields.fields.userfield.UserField(
                blank=True, editable=False, max_length=50, verbose_name="user modified"
            ),
        ),
        migrations.AddField(
            model_name="identifierhistory",
            name="created",
            field=models.DateTimeField(default=edc_utils.date.get_utcnow, editable=False),
        ),
        migrations.AddField(
            model_name="identifierhistory",
            name="hostname_created",
            field=models.CharField(
                default="mac2-2.local",
                editable=False,
                help_text="System field. (modified on create only)",
                max_length=50,
            ),
        ),
        migrations.AddField(
            model_name="identifierhistory",
            name="hostname_modified",
            field=edc_model_fields.fields.hostname_modification_field.HostnameModificationField(
                blank=True,
                editable=False,
                help_text="System field. (modified on every save)",
                max_length=50,
            ),
        ),
        migrations.AddField(
            model_name="identifierhistory",
            name="modified",
            field=models.DateTimeField(default=edc_utils.date.get_utcnow, editable=False),
        ),
        migrations.AddField(
            model_name="identifierhistory",
            name="revision",
            field=django_revision.revision_field.RevisionField(
                blank=True,
                editable=False,
                help_text="System field. Git repository tag:branch:commit.",
                max_length=75,
                null=True,
                verbose_name="Revision",
            ),
        ),
        migrations.AddField(
            model_name="identifierhistory",
            name="user_created",
            field=edc_model_fields.fields.userfield.UserField(
                blank=True, editable=False, max_length=50, verbose_name="user created"
            ),
        ),
        migrations.AddField(
            model_name="identifierhistory",
            name="user_modified",
            field=edc_model_fields.fields.userfield.UserField(
                blank=True, editable=False, max_length=50, verbose_name="user modified"
            ),
        ),
        migrations.AlterField(
            model_name="historicalidentifierhistory",
            name="history_id",
            field=django_audit_fields.fields.uuid_auto_field.UUIDAutoField(
                primary_key=True, serialize=False
            ),
        ),
        migrations.AlterField(
            model_name="historicalidentifierhistory",
            name="id",
            field=django_audit_fields.fields.uuid_auto_field.UUIDAutoField(
                blank=True,
                db_index=True,
                editable=False,
                help_text="System auto field. UUID primary key.",
            ),
        ),
        migrations.AlterField(
            model_name="historicalidentifiertracker",
            name="created",
            field=models.DateTimeField(default=edc_utils.date.get_utcnow, editable=False),
        ),
        migrations.AlterField(
            model_name="historicalidentifiertracker",
            name="modified",
            field=models.DateTimeField(default=edc_utils.date.get_utcnow, editable=False),
        ),
        migrations.AlterField(
            model_name="historicalsubjectidentifier",
            name="created",
            field=models.DateTimeField(default=edc_utils.date.get_utcnow, editable=False),
        ),
        migrations.AlterField(
            model_name="historicalsubjectidentifier",
            name="modified",
            field=models.DateTimeField(default=edc_utils.date.get_utcnow, editable=False),
        ),
        migrations.AlterField(
            model_name="identifierhistory",
            name="id",
            field=django_audit_fields.fields.uuid_auto_field.UUIDAutoField(
                blank=True,
                editable=False,
                help_text="System auto field. UUID primary key.",
                primary_key=True,
                serialize=False,
            ),
        ),
        migrations.AlterField(
            model_name="identifiertracker",
            name="created",
            field=models.DateTimeField(default=edc_utils.date.get_utcnow, editable=False),
        ),
        migrations.AlterField(
            model_name="identifiertracker",
            name="modified",
            field=models.DateTimeField(default=edc_utils.date.get_utcnow, editable=False),
        ),
        migrations.AlterField(
            model_name="sequence",
            name="created",
            field=models.DateTimeField(default=edc_utils.date.get_utcnow, editable=False),
        ),
        migrations.AlterField(
            model_name="sequence",
            name="modified",
            field=models.DateTimeField(default=edc_utils.date.get_utcnow, editable=False),
        ),
        migrations.AlterField(
            model_name="subjectidentifier",
            name="created",
            field=models.DateTimeField(default=edc_utils.date.get_utcnow, editable=False),
        ),
        migrations.AlterField(
            model_name="subjectidentifier",
            name="modified",
            field=models.DateTimeField(default=edc_utils.date.get_utcnow, editable=False),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.9 on 2016-05-04 11:10
from __future__ import unicode_literals

import django.utils.timezone
import django_audit_fields.fields.uuid_auto_field
import django_extensions.db.fields
import django_revision.revision_field
import edc_model_fields.fields.hostname_modification_field
import edc_model_fields.fields.userfield
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = []

    operations = [
        migrations.CreateModel(
            name="IdentifierHistory",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("identifier", models.CharField(max_length=25, unique=True)),
                ("identifier_type", models.CharField(max_length=25)),
                ("identifier_prefix", models.CharField(max_length=25, null=True)),
                (
                    "created_datetime",
                    models.DateTimeField(default=django.utils.timezone.now),
                ),
            ],
        ),
        migrations.CreateModel(
            name="IdentifierTracker",
            fields=[
                (
                    "created",
                    django_extensions.db.fields.CreationDateTimeField(
                        auto_now_add=True, verbose_name="created"
                    ),
                ),
                (
                    "modified",
                    django_extensions.db.fields.ModificationDateTimeField(
                        auto_now=True, verbose_name="modified"
                    ),
                ),
                (
                    "user_created",
                    edc_model_fields.fields.userfield.UserField(
                        editable=False, max_length=50, verbose_name="user created"
                    ),
                ),
                (
                    "user_modified",
                    edc_model_fields.fields.userfield.UserField(
                        editable=False, max_length=50, verbose_name="user modified"
                    ),
                ),
                (
                    "hostname_created",
                    models.CharField(
                        default="One-2.local",
                        editable=False,
                        help_text="System field. (modified on create only)",
                        max_length=50,
                    ),
                ),
                (
                    "hostname_modified",
                    edc_model_fields.fields.hostname_modification_field.HostnameModificationField(
                        editable=False,
                        help_text="System field. (modified on every save)",
                        max_length=50,
                    ),
                ),
                (
                    "revision",
                    django_revision.revision_field.RevisionField(
                        blank=True,
                        editable=False,
                        help_text="System field. Git repository tag:branch:commit.",
                        max_length=75,
                        null=True,
                        verbose_name="Revision",
                    ),
                ),
                (
                    "id",
                    django_audit_fields.fields.uuid_auto_field.UUIDAutoField(
                        editable=False,
                        help_text="System field. UUID primary key.",
                        primary_key=True,
                        serialize=False,
                    ),
                ),
                ("identifier", models.CharField(db_index=True, max_length=25)),
                ("identifier_string", models.CharField(db_index=True, max_length=50)),
                ("root_number", models.IntegerField(db_index=True)),
                ("counter", models.IntegerField(db_index=True)),
                ("identifier_type", models.CharField(max_length=35)),
                ("device_id", models.CharField(blank=True, max_length=10, null=True)),
            ],
            options={"ordering": ["root_number", "counter"]},
        ),
        migrations.CreateModel(
            name="Sequence",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "created",
                    django_extensions.db.fields.CreationDateTimeField(
                        auto_now_add=True, verbose_name="created"
                    ),
                ),
                (
                    "modified",
                    django_extensions.db.fields.ModificationDateTimeField(
                        auto_now=True, verbose_name="modified"
                    ),
                ),
                (
                    "user_created",
                    edc_model_fields.fields.userfield.UserField(
                        editable=False, max_length=50, verbose_name="user created"
                    ),
                ),
                (
                    "user_modified",
                    edc_model_fields.fields.userfield.UserField(
                        editable=False, max_length=50, verbose_name="user modified"
                    ),
                ),
                (
                    "hostname_created",
                    models.CharField(
                        default="One-2.local",
                        editable=False,
                        help_text="System field. (modified on create only)",
                        max_length=50,
                    ),
                ),
                (
                    "hostname_modified",
                    edc_model_fields.fields.hostname_modification_field.HostnameModificationField(
                        editable=False,
                        help_text="System field. (modified on every save)",
                        max_length=50,
                    ),
                ),
                (
                    "revision",
                    django_revision.revision_field.RevisionField(
                        blank=True,
                        editable=False,
                        help_text="System field. Git repository tag:branch:commit.",
                        max_length=75,
                        null=True,
                        verbose_name="Revision",
                    ),
                ),
                ("device_id", models.IntegerField(default=99)),
            ],
            options={"ordering": ["id"]},
        ),
        migrations.CreateModel(
            name="SubjectIdentifier",
            fields=[
                (
                    "created",
                    django_extensions.db.fields.CreationDateTimeField(
                        auto_now_add=True, verbose_name="created"
                    ),
                ),
                (
                    "modified",
                    django_extensions.db.fields.ModificationDateTimeField(
                        auto_now=True, verbose_name="modified"
                    ),
                ),
                (
                    "user_created",
                    edc_model_fields.fields.userfield.UserField(
                        editable=False, max_length=50, verbose_name="user created"
                    ),
                ),
                (
                    "user_modified",
                    edc_model_fields.fields.userfield.UserField(
                        editable=False, max_length=50, verbose_name="user modified"
                    ),
                ),
                (
                    "hostname_created",
                    models.CharField(
                        default="One-2.local",
                        editable=False,
                        help_text="System field. (modified on create only)",
                        max_length=50,
                    ),
                ),
                (
                    "hostname_modified",
                    edc_model_fields.fields.hostname_modification_field.HostnameModificationField(
                        editable=False,
                        help_text="System field. (modified on every save)",
                        max_length=50,
                    ),
                ),
                (
                    "revision",
                    django_revision.revision_field.RevisionField(
                        blank=True,
                        editable=False,
                        help_text="System field. Git repository tag:branch:commit.",
                        max_length=75,
                        null=True,
                        verbose_name="Revision",
                    ),
                ),
                (
                    "id",
                    django_audit_fields.fields.uuid_auto_field.UUIDAutoField(
                        editable=False,
                        help_text="System field. UUID primary key.",
                        primary_key=True,
                        serialize=False,
                    ),
                ),
                (
                    "identifier",
                    models.CharField(editable=False, max_length=36, unique=True),
                ),
                ("padding", models.IntegerField(default=4, editable=False)),
                ("sequence_number", models.IntegerField()),
                ("device_id", models.IntegerField(default=0)),
                ("is_derived", models.BooleanField(default=False)),
                (
                    "sequence_app_label",
                    models.CharField(default="identifier", editable=False, max_length=50),
                ),
                (
                    "sequence_model_name",
                    models.CharField(default="sequence", editable=False, max_length=50),
                ),
            ],
            options={"ordering": ["-created"]},
        ),
        migrations.AlterUniqueTogether(
            name="identifiertracker", unique_together=set([("root_number", "counter")])
        ),
    ]



from django.test import TestCase

from edc_identifier.checkdigit_mixins import LuhnMixin, LuhnOrdMixin
from edc_identifier.identifier import Identifier


class TestIdentifier(TestCase):
    def test_valid_checkdigit(self):
        mixin = LuhnMixin()
        checkdigit = mixin.calculate_checkdigit("98765")
        self.assertEqual(checkdigit, "1")
        checkdigit = mixin.calculate_checkdigit("98766")
        self.assertEqual(checkdigit, "9")
        checkdigit = mixin.calculate_checkdigit("98767")
        self.assertEqual(checkdigit, "7")

    def test_luhn_ord_mixin(self):
        mixin = LuhnOrdMixin()
        self.assertEqual("4", mixin.calculate_checkdigit("ABCDE"))
        self.assertEqual("8", mixin.calculate_checkdigit("ABCDEF"))
        self.assertEqual("0", mixin.calculate_checkdigit("ABCDEFG"))

    def test_identifier(self):
        instance = Identifier()
        self.assertEqual("1", instance.identifier)
        instance = Identifier()
        self.assertEqual("2", instance.identifier)



# -*- coding: utf-8 -*-
# Generated by Django 1.11.1 on 2017-05-11 13:23
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [("edc_identifier", "0010_auto_20170112_0602")]

    operations = [
        migrations.RemoveField(
            model_name="historicalidentifierhistory", name="created_datetime"
        ),
        migrations.RemoveField(model_name="identifierhistory", name="created_datetime"),
        migrations.AlterField(
            model_name="historicalidentifierhistory",
            name="identifier",
            field=models.CharField(db_index=True, max_length=50),
        ),
        migrations.AlterField(
            model_name="historicalidentifierhistory",
            name="identifier_type",
            field=models.CharField(max_length=50),
        ),
        migrations.AlterField(
            model_name="identifierhistory",
            name="identifier",
            field=models.CharField(max_length=50, unique=True),
        ),
        migrations.AlterField(
            model_name="identifierhistory",
            name="identifier_type",
            field=models.CharField(max_length=50),
        ),
    ]



from django.apps import apps as django_apps
from django.core.exceptions import ObjectDoesNotExist
from django.test import TestCase
from faker import Faker

from edc_identifier.models import IdentifierModel
from edc_identifier.short_identifier import (
    DuplicateIdentifierError,
    ShortIdentifier,
    ShortIdentifierPrefixError,
    ShortIdentifierPrefixPatternError,
)

fake = Faker()


class TestShortIdentifier(TestCase):
    def setUp(self):
        edc_device_app_config = django_apps.get_app_config("edc_device")
        self.device_id = edc_device_app_config.device_id

    def test_short_identifier(self):
        short_identifier = ShortIdentifier(prefix_pattern="^[0-9]{2}$", prefix=22)
        self.assertIsNotNone(short_identifier.identifier)

    def test_short_identifier_str(self):
        short_identifier = ShortIdentifier(prefix_pattern="^[0-9]{2}$", prefix=22)
        self.assertEqual(str(short_identifier), short_identifier.identifier)

    def test_short_identifier_invalid_prefix_pattern(self):
        self.assertRaises(
            ShortIdentifierPrefixPatternError,
            ShortIdentifier,
            prefix_pattern="[0-9]{2}",
            prefix=22,
        )

    def test_short_identifier_invalid_prefix(self):
        self.assertRaises(
            ShortIdentifierPrefixError,
            ShortIdentifier,
            prefix_pattern="^[0-9]{2}$",
            prefix="AA",
        )

    def test_short_identifier_needs_prefix_if_pattern(self):
        self.assertRaises(
            ShortIdentifierPrefixError,
            ShortIdentifier,
            prefix_pattern="^[0-9]{2}$",
            prefix=None,
        )

    def test_short_identifier_needs_prefix_pattern_if_prefix(self):
        class NewCls(ShortIdentifier):
            prefix_pattern = None

        self.assertRaises(ShortIdentifierPrefixError, NewCls, prefix_pattern=None, prefix=22)

    def test_short_identifier_prefix_pattern_must_match_prefix(self):
        self.assertRaises(
            ShortIdentifierPrefixError,
            ShortIdentifier,
            prefix_pattern="^[0-9]{2}$",
            prefix="AA",
        )

    def test_short_identifier_identifier_model(self):
        short_identifier = ShortIdentifier(prefix_pattern="^[0-9]{2}$", prefix=22)
        self.assertEqual(str(short_identifier), short_identifier.identifier)

    def test_short_identifier_longer_than_default(self):
        prefix = "222"
        random_string_length = 7
        prefix_pattern = "^[0-9]{3}$"
        short_identifier = ShortIdentifier(
            prefix_pattern=prefix_pattern,
            prefix=prefix,
            random_string_length=random_string_length,
        )
        self.assertEqual(len(short_identifier.identifier), 10)

    def test_short_identifier_history(self):
        short_identifier = ShortIdentifier(prefix_pattern="^[0-9]{2}$", prefix=22)
        try:
            IdentifierModel.objects.get(identifier=short_identifier.identifier),
        except ObjectDoesNotExist:
            self.fail("ObjectDoesNotExist unexpectedly raised")

    def test_short_identifier_with_last1(self):

        prefix = "22"
        IdentifierModel.objects.create(
            identifier="22KVTB4",
            identifier_type=ShortIdentifier.name,
            identifier_prefix=prefix,
            device_id=self.device_id,
        )

        short_identifier = ShortIdentifier(prefix_pattern="^[0-9]{2}$", prefix=prefix)
        self.assertNotEqual(short_identifier.identifier, "22KVTB4")

    def test_short_identifier_with_last2(self):

        prefix = "22"
        random_string_length = 5
        prefix_pattern = "^[0-9]{2}$"

        IdentifierModel.objects.create(
            identifier="22KVTB4",
            identifier_type=ShortIdentifier.name,
            identifier_prefix=prefix,
            device_id=self.device_id,
        )

        short_identifier = ShortIdentifier(
            prefix_pattern=prefix_pattern,
            prefix=prefix,
            random_string_length=random_string_length,
        )

        self.assertEqual(len(short_identifier.identifier), 7)

    def test_short_identifier_catches_duplicate_limit1(self):
        """Asserts raises if a duplicate identifier is generated."""
        options = dict(random_string_pattern=r"[AB]+", random_string_length=3)
        n = 1
        tries = 100
        while n < tries:
            n += 1
            try:
                ShortIdentifier(prefix="22", **options)
            except DuplicateIdentifierError:
                break
        self.assertGreater(n, 7)
        self.assertLess(n, 11)

    def test_short_identifier_catches_duplicate_limit2(self):
        """Asserts raises if a duplicate identifier is generated."""
        options = dict(random_string_pattern=r"[AB]+", random_string_length=5)
        n = 1
        tries = 100
        while n < tries:
            n += 1
            try:
                ShortIdentifier(prefix="22", **options)
            except DuplicateIdentifierError:
                break
        self.assertGreater(n, 32)
        self.assertLess(n, 35)



from django.core.exceptions import ObjectDoesNotExist
from django.test import TestCase

from edc_identifier.models import IdentifierModel
from edc_identifier.simple_identifier import SimpleIdentifier, SimpleUniqueIdentifier


class TestSimpleIdentifier(TestCase):
    def test_simple(self):
        obj = SimpleIdentifier()
        obj.identifier

    def test_simple_unique(self):
        obj = SimpleUniqueIdentifier()
        self.assertIsNotNone(obj.identifier)
        try:
            IdentifierModel.objects.get(identifier=obj.identifier)
        except ObjectDoesNotExist:
            self.fail("Identifier not add to history")



from unittest.case import skip

from django.test import TestCase
from faker import Faker

from edc_identifier.exceptions import IdentifierError, SubjectIdentifierError
from edc_identifier.models import IdentifierModel
from edc_identifier.subject_identifier import SubjectIdentifier

from .models import Enrollment, EnrollmentThree

fake = Faker()


class TestSubjectIdentifier(TestCase):
    def test_create(self):
        """Asserts raises exception if cannot find cap."""
        subject_identifier = SubjectIdentifier(
            identifier_type="subject",
            requesting_model="edc_identifier.enrollment",
            protocol_number="000",
            device_id="99",
        )
        self.assertTrue(subject_identifier.identifier)

    @skip("enrollment cap not implemented")
    def test_raises_on_unknown_cap(self):
        """Asserts raises exception if cannot find cap."""
        self.assertRaises(
            SubjectIdentifierError,
            SubjectIdentifier,
            identifier_type="subjectblahblah",
            requesting_model="edc_identifier.enrollmentblahblahblah",
            protocol_number="000",
            device_id="99",
        )

    def test_increments(self):
        """Asserts identifier sequence increments correctly."""
        opts = dict(identifier_type="subject", requesting_model="edc_identifier.enrollment")
        for i in range(1, 10):
            subject_identifier = SubjectIdentifier(**opts)
            self.assertEqual(subject_identifier.identifier[7:11], "000" + str(i))

    def test_create_missing_args(self):
        """Asserts raises exception for missing identifier_type."""
        self.assertRaises(
            IdentifierError,
            SubjectIdentifier,
            identifier_type="",
            requesting_model="edc_identifier.enrollment",
        )

    def test_create_missing_args2(self):
        """Asserts raises exception for missing model."""
        self.assertRaises(
            IdentifierError,
            SubjectIdentifier,
            identifier_type="subject",
            requesting_model="",
        )

    def test_create1(self):
        """Asserts exact first identifier given parameters."""
        subject_identifier = SubjectIdentifier(
            identifier_type="subject",
            requesting_model="edc_identifier.enrollment",
            protocol_number="000",
            device_id="99",
        )
        self.assertEqual("000-1990001-8", subject_identifier.identifier)

    def test_create2(self):
        """Asserts exact first identifier required parameters
        and those fetched from edc-example.AppConfig.
        """
        subject_identifier = SubjectIdentifier(
            identifier_type="subject",
            requesting_model="edc_identifier.enrollment",
            protocol_number="000",
        )
        self.assertEqual("000-1140001-7", subject_identifier.identifier)

    @skip("enrollment cap not implemented")
    def test_create_hits_cap(self):
        """Asserts raises exception if attempt to exceed cap."""
        for _ in range(1, 6):
            identifier = SubjectIdentifier(
                identifier_type="subject",
                requesting_model="edc_identifier.enrollmentthree",
                protocol_number="000",
                device_id="99",
            )
            EnrollmentThree.objects.create(subject_identifier=identifier.identifier)
            self.assertIsNotNone(identifier.identifier)
        self.assertEqual(IdentifierModel.objects.all().count(), 5)
        self.assertRaises(
            Exception,
            SubjectIdentifier,
            identifier_type="subject",
            requesting_model="edc_identifier.enrollmentthree",
            protocol_number="000",
            device_id="99",
        )

    @skip("enrollment cap not implemented")
    def test_create_hits_cap_with_other_models(self):
        """Asserts raises exception if attempt to exceed cap."""
        for _ in range(0, 10):
            identifier = SubjectIdentifier(
                identifier_type="subject",
                requesting_model="edc_identifier.enrollment",
                protocol_number="000",
                device_id="99",
            )
            Enrollment.objects.create(subject_identifier=identifier.identifier)
            self.assertIsNotNone(identifier.identifier)
        for _ in range(0, 5):
            identifier = SubjectIdentifier(
                identifier_type="subject",
                requesting_model="edc_identifier.enrollmentthree",
                protocol_number="000",
                device_id="99",
            )
            EnrollmentThree.objects.create(subject_identifier=identifier.identifier)
            self.assertIsNotNone(identifier.identifier)
        self.assertEqual(
            IdentifierModel.objects.filter(model="edc_identifier.enrollmentthree").count(),
            5,
        )
        self.assertRaises(
            Exception,
            SubjectIdentifier,
            identifier_type="subject",
            requesting_model="edc_identifier.enrollmentthree",
            protocol_number="000",
            device_id="99",
        )

    def test_updates_identifier_model(self):
        """Asserts updates Identifier model with all attributes."""
        for _ in range(0, 5):
            identifier = SubjectIdentifier(
                identifier_type="subject",
                requesting_model="edc_identifier.enrollmentthree",
                protocol_number="000",
                device_id="99",
            )
            EnrollmentThree.objects.create(subject_identifier=identifier.identifier)
            self.assertIsNotNone(identifier.identifier)
        self.assertEqual(IdentifierModel.objects.all().count(), 5)
        self.assertEqual(
            IdentifierModel.objects.filter(
                identifier_type="subject",
                model="edc_identifier.enrollmentthree",
                protocol_number="000",
                device_id="99",
            ).count(),
            5,
        )
